@@ -159,6 +159,19 @@ function os._rm(filedir)
     return true
 end
 
+-- remove empty parent directories of this file path
+function os._rm_empty_parentdirs(filepath)
+    local parentdir = path.directory(filepath)
+    while parentdir and os.isdir(parentdir) and os.emptydir(parentdir) do
+        local ok, errors = os._rm(parentdir)
+        if not ok then
+            return false, errors
+        end
+        parentdir = path.directory(parentdir)
+    end
+    return true
+end
+
 -- get the ramdisk root directory
 -- https://github.com/xmake-io/xmake/issues/3408
 function os._ramdir()
@@ -317,7 +330,7 @@ function os.match(pattern, mode, callback)
     end
 
     -- translate path and remove some repeat separators
-    pattern = path.translate(pattern:gsub("|.*$", ""))
+    pattern = path.translate((pattern:gsub("|.*$", "")))
 
     -- translate mode
     if type(mode) == "string" then
@@ -464,7 +477,7 @@ function os.mv(srcpath, dstpath, opt)
 end
 
 -- remove files or directories
-function os.rm(filepath)
+function os.rm(filepath, opt)
 
     -- check arguments
     if not filepath then
@@ -472,16 +485,29 @@ function os.rm(filepath)
     end
 
     -- remove file or directories
+    opt = opt or {}
     filepath = tostring(filepath)
     local filepathes = os._match_wildcard_pathes(filepath)
     if type(filepathes) == "string" then
-        return os._rm(filepathes)
+        local ok, errors = os._rm(filepathes)
+        if not ok then
+            return false, errors
+        end
+        if opt.emptydirs then
+            return os._rm_empty_parentdirs(filepathes)
+        end
     else
         for _, _filepath in ipairs(filepathes) do
             local ok, errors = os._rm(_filepath)
             if not ok then
                 return false, errors
             end
+            if opt.emptydirs then
+                ok, errors = os._rm_empty_parentdirs(filepath)
+                if not ok then
+                    return false, errors
+                end
+            end
         end
     end
     return true
@@ -701,7 +727,7 @@ function os.runv(program, argv, opt)
     local logfile = os.tmpfile()
 
     -- execute it
-    local ok, errors = os.execv(program, argv, table.join(opt, {stdout = logfile, stderr = logfile}))
+    local ok, errors = os.execv(program, argv, table.join(opt, {stdout = opt.stdout or logfile, stderr = opt.stderr or logfile}))
     if ok ~= 0 then
 
         -- get command
@@ -776,25 +802,27 @@ function os.execv(program, argv, opt)
     if opt.shell and os.isfile(filename) then
         local shellfile = filename
         local file = io.open(filename, 'r')
-        local head = file:read("l")
-        if head and head:startswith("#!") then
-            -- we cannot run `/bin/sh` directly on msys2/cygwin
-            -- because `/bin/sh` is not real file path, maybe we need to convert it.
-            local subhost = os.subhost()
-            if subhost == "msys" or subhost == "cygwin" then
-                filename = "sh"
-                argv = table.join(shellfile, argv)
-            else
-                head = head:sub(3)
-                local shellargv = {}
-                local splitinfo = head:split("%s")
-                filename = splitinfo[1]
-                if #splitinfo > 1 then
-                    shellargv = table.slice(splitinfo, 2)
+        for line in file:lines() do
+            if line and line:startswith("#!") then
+                -- we cannot run `/bin/sh` directly on windows
+                -- because `/bin/sh` is not real file path, maybe we need to convert it.
+                local host = os.host()
+                if host == "windows" then
+                    filename = "sh"
+                    argv = table.join(shellfile, argv)
+                else
+                    line = line:sub(3)
+                    local shellargv = {}
+                    local splitinfo = line:split("%s")
+                    filename = splitinfo[1]
+                    if #splitinfo > 1 then
+                        shellargv = table.slice(splitinfo, 2)
+                    end
+                    table.insert(shellargv, shellfile)
+                    table.join2(shellargv, argv)
+                    argv = shellargv
                 end
-                table.insert(shellargv, shellfile)
-                table.join2(shellargv, argv)
-                argv = shellargv
+                break
             end
         end
         file:close()
@@ -832,6 +860,7 @@ function os.execv(program, argv, opt)
 
     -- open command
     local ok = -1
+    local errors
     local proc = process.openv(filename, argv or {}, openopt)
     if proc ~= nil then
 
@@ -843,9 +872,16 @@ function os.execv(program, argv, opt)
 
         -- wait process
         if not opt.detach then
-            local waitok, status = proc:wait(-1)
+            local waitok, status = proc:wait(opt.timeout or -1)
             if waitok > 0 then
                 ok = status
+            elseif waitok == 0 and opt.timeout then
+                proc:kill()
+                waitok, status = proc:wait(-1)
+                if waitok > 0 then
+                    ok = status
+                end
+                errors = "wait process timeout"
             end
         else
             ok = 0
@@ -857,7 +893,7 @@ function os.execv(program, argv, opt)
         -- cannot execute process
         return nil, os.strerror()
     end
-    return ok
+    return ok, errors
 end
 
 -- run command and return output and error data
