@@ -76,6 +76,7 @@ rule("qt.console")
     end)
 
     after_install("windows", "install.windows")
+    after_install("mingw", "install.mingw")
 
 -- define rule: qt widgetapp
 rule("qt.widgetapp")
@@ -110,6 +111,7 @@ rule("qt.widgetapp")
     -- install application for android
     on_install("android", "install.android")
     after_install("windows", "install.windows")
+    after_install("mingw", "install.mingw")
 
 -- define rule: qt static widgetapp
 rule("qt.widgetapp_static")
@@ -121,50 +123,11 @@ rule("qt.widgetapp_static")
     end)
 
     on_config(function (target)
-
-        -- get qt sdk version
-        local qt = target:data("qt")
-        local qt_sdkver = nil
-        if qt.sdkver then
-            import("core.base.semver")
-            qt_sdkver = semver.new(qt.sdkver)
-        end
-
-        -- @see
-        -- https://github.com/xmake-io/xmake/issues/1047
-        -- https://github.com/xmake-io/xmake/issues/2791
-        local QtPlatformSupport
-        if qt_sdkver then
-            if qt_sdkver:ge("6.0") then
-                QtPlatformSupport = nil
-            elseif qt_sdkver:ge("5.9") then
-                QtPlatformSupport = "QtPlatformCompositorSupport"
-            else
-                QtPlatformSupport = "QtPlatformSupport"
-            end
-        end
-
-        -- load some basic plugins and frameworks
-        local plugins = {}
-        local frameworks = {"QtGui", "QtWidgets", "QtCore"}
-        if qt_sdkver and qt_sdkver:lt("5.0") then
-            frameworks = {"QtGui", "QtCore"} -- qt4.x has not QtWidgets, it is in QtGui
-        end
-        if target:is_plat("macosx") then
-            plugins.QCocoaIntegrationPlugin = {linkdirs = "plugins/platforms", links = {"qcocoa", "cups"}}
-            table.insert(frameworks, "QtWidgets")
-            if QtPlatformSupport then
-                table.insert(frameworks, QtPlatformSupport)
-            end
-        elseif target:is_plat("windows") then
-            plugins.QWindowsIntegrationPlugin = {linkdirs = "plugins/platforms", links = {is_mode("debug") and "qwindowsd" or "qwindows"}}
-            table.join2(frameworks, "QtPrintSupport", "QtWidgets")
-            if QtPlatformSupport then
-                table.insert(frameworks, QtPlatformSupport)
-            end
-        elseif target:is_plat("wasm") then
-            plugins.QWasmIntegrationPlugin = {linkdirs = "plugins/platforms", links = {"qwasm"}}
-            table.join2(frameworks, "QtEventDispatcherSupport", "QtFontDatabaseSupport", "QtEglSupport")
+        local frameworks, plugins, qt_sdkver = import("config_static")(target)
+        if qt_sdkver:ge("5.0") then
+            table.join2(frameworks, {"QtGui", "QtWidgets", "QtCore"})
+        else
+            table.join2(frameworks, {"QtGui", "QtCore"})-- qt4.x has not QtWidgets, it is in QtGui
         end
         import("load")(target, {gui = true, plugins = plugins, frameworks = frameworks})
     end)
@@ -176,6 +139,7 @@ rule("qt.widgetapp_static")
     -- install application for android
     on_install("android", "install.android")
     after_install("windows", "install.windows")
+    after_install("mingw", "install.mingw")
 
 -- define rule: qt quickapp
 rule("qt.quickapp")
@@ -197,6 +161,7 @@ rule("qt.quickapp")
     -- install application for android
     on_install("android", "install.android")
     after_install("windows", "install.windows")
+    after_install("mingw", "install.mingw")
 
 -- define rule: qt static quickapp
 rule("qt.quickapp_static")
@@ -208,48 +173,8 @@ rule("qt.quickapp_static")
     end)
 
     on_config(function (target)
-
-        -- get qt sdk version
-        local qt = target:data("qt")
-        local qt_sdkver = nil
-        if qt.sdkver then
-            import("core.base.semver")
-            qt_sdkver = semver.new(qt.sdkver)
-        end
-
-        -- @see
-        -- https://github.com/xmake-io/xmake/issues/1047
-        -- https://github.com/xmake-io/xmake/issues/2791
-        local QtPlatformSupport
-        if qt_sdkver then
-            if qt_sdkver:ge("6.0") then
-                QtPlatformSupport = nil
-            elseif qt_sdkver:ge("5.9") then
-                QtPlatformSupport = "QtPlatformCompositorSupport"
-            else
-                QtPlatformSupport = "QtPlatformSupport"
-            end
-        end
-
-        -- load some basic plugins and frameworks
-        local plugins = {}
-        local frameworks = {"QtGui", "QtQuick", "QtQml", "QtQmlModels", "QtCore", "QtNetwork"}
-        if target:is_plat("macosx") then
-            plugins.QCocoaIntegrationPlugin = {linkdirs = "plugins/platforms", links = {"qcocoa", "cups"}}
-            table.insert(frameworks, "QtWidgets")
-            if QtPlatformSupport then
-                table.insert(frameworks, QtPlatformSupport)
-            end
-        elseif target:is_plat("windows") then
-            plugins.QWindowsIntegrationPlugin = {linkdirs = "plugins/platforms", links = {is_mode("debug") and "qwindowsd" or "qwindows"}}
-            table.join2(frameworks, "QtPrintSupport", "QtWidgets")
-            if QtPlatformSupport then
-                table.insert(frameworks, QtPlatformSupport)
-            end
-        elseif target:is_plat("wasm") then
-            plugins.QWasmIntegrationPlugin = {linkdirs = "plugins/platforms", links = {"qwasm"}}
-            table.join2(frameworks, "QtEventDispatcherSupport", "QtFontDatabaseSupport", "QtEglSupport")
-        end
+        local frameworks, plugins = import("config_static")(target)
+        table.join2(frameworks, {"QtGui", "QtQuick", "QtQml", "QtQmlModels", "QtCore", "QtNetwork"})
         import("load")(target, {gui = true, plugins = plugins, frameworks = frameworks})
     end)
 
@@ -260,11 +185,12 @@ rule("qt.quickapp_static")
     -- install application for android
     on_install("android", "install.android")
     after_install("windows", "install.windows")
+    after_install("mingw", "install.mingw")
 
 -- define rule: qt qmlplugin
 rule("qt.qmlplugin")
     add_deps("qt.shared", "qt.qmltyperegistrar")
-    on_load(function(target)
+    on_config(function(target)
         import("load")(target, {frameworks = { "QtCore", "QtGui", "QtQuick", "QtQml", "QtNetwork" }})
     end)
 
